<?php

/*
Plugin Name: Woocommerce Cylcos Payment
Description: Plugin to enable Cyclos as a payment gateway in Woocommerce.
Version: 1.0.23
Author: EXO Designs
Author URI: https://exodesigns.nl/
Plugin URI: https://exodesigns.nl/
Text Domain: EXO
WC requires at least: 3.0.0
WC tested up to: 3.6.2
License: GPL3
*/

/* Copyright (C) 1883 Thomas Edison - All Rights Reserved
 * You may use, distribute and modify this code under the
 * terms of the GNU GPl v3 license, which unfortunately won't be
 * written for another century.
 *
 * You should have received a copy of the GNU GPl v3 license with
 * this file. If not, please contact the developer at: https://exodesigns.nl
 */

add_filter('plugin_action_links_'.plugin_basename(__FILE__), 'exo_add_plugin_page_settings_link');
function exo_add_plugin_page_settings_link( $links ) {
	$links[] = '<a href="' .
		admin_url( 'admin.php?page=wc-settings&tab=checkout&section=cyclos' ) .
		'">' . __('Payment Settings') . '</a>';
	return $links;
}

function debug_to_console( $data ) {
    $output = $data;
    if ( is_array( $output ) )
        $output = implode( ',', $output);

    echo "<script>console.log( 'Debug Objects: " . $output . "' );</script>";
}

add_filter( 'woocommerce_payment_gateways', 'cyclos_add_gateway_class' );
function cyclos_add_gateway_class( $gateways ) {
	$gateways[] = 'WC_Cyclos_Gateway'; 
	return $gateways;
}
 
//Class itself
add_action( 'plugins_loaded', 'cyclos_init_gateway_class' );
function cyclos_init_gateway_class() {

	class WC_Cyclos_Gateway extends WC_Payment_Gateway {
 
		//Constructor
 		public function __construct() {
 
		$this->id = 'cyclos'; // Payment Gateway Plugin ID
		$this->icon = ''; // URL of the icon, this will be displayed on checkout page near your gateway name
		$this->has_fields = false; // No creditcard fields during payment
		$this->method_title = 'Cyclos Payments'; // Payment Gateway Title, will be displayed in options page
		$this->method_description = 'Enable payment through Cyclos'; // Payment Gateway Description, will be displayed on the options page
	 
		// gateways can support subscriptions, refunds, saved payment methods,
		// This payment gateway is just a transaction
		$this->supports = array(
			'products'
		);
	 
		// Method with all the options fields
		$this->init_form_fields();

		// Load the settings
		$this->init_settings();
		$this->title = $this->get_option( 'title' );
		$this->description = $this->get_option( 'description' );
		$this->enabled = $this->get_option( 'enabled' );
		$this->testmode = 'yes' === $this->get_option( 'testmode' );
		$this->root_url = $this->testmode ? $this->get_option( 'test_root_url' ) : $this->get_option( 'root_url' );
		$this->api_username = $this->testmode ? $this->get_option( 'test_api_username' ) : $this->get_option( 'api_username' );
		$this->api_password = $this->testmode ? $this->get_option( 'test_api_password' ) : $this->get_option( 'api_password' );
		$this->succes_url = $this->get_option('succes_url');
		$this->cancel_url = $this->get_option('cancel_url');
		$this->accessclientmode = 'yes' === $this->get_option( 'accessclientmode' );
		$this->api_accessclient = $this->accessclientmode ? ($this->testmode ? $this->get_option( 'test_api_accessclient' ) : $this->get_option( 'api_accessclient' )) : NULL;
		// Globalize API config
		$GLOBALS['config'] = [
		    // The root URL of the Cyclo server / network The authentication should either be with access client token or user / password.

		    'root' => $this->root_url,
		    
		    // To authenticate with an access client, set it here. Otherwise, leave NULL
		    'accessClient' => $this->api_accessclient,
		    
		    // To authenticate with user / password, set both here (and make sure accessClient is NULL). Otherwise, leave NULL
		    'user' => $this->api_username,
		    'password' => $this->api_password
		];
	 	
	 	//Globalize payment Title
	 	$GLOBALS['payment_title'] = $this->title;

	 	//Webhook for redirecting to payment_return function when payment is complete.
	 	add_action( 'woocommerce_api_cyclos_payment_completed', array( $this, 'payment_returns' ) );
	 	
		// This action hook saves the admin settings
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
	 
		// hook to create a payment token
		add_action( 'wp_enqueue_scripts', array( $this, 'payment_scripts' ) );
 
 		//Check if accessClient token is being generated, if so then show result, regarding if it is an error or not.
		 if(!empty($_POST['accessClientCode'])){
		   	$accesscode = $_POST['accessClientCode']; 	
		   	$token = get_accessclient($accesscode);
		   	$GLOBALS['GeneratedAccessClientToken'] = $token;
		 }

 		}

 		//Webhook function when payment is complete.

 		public function payment_returns() {
 
			echo "OK";
			$order = wc_get_order( $_GET['orderId'] );
			$orderId = $_GET['orderId'];
			echo "<br>OrderID: $orderId";
			$ticketNumber = $_GET['ticketNumber'];
		 
			//process-payment in Cyclos Enviroment

			try {
			    $transactionNumber = process_ticket($ticketNumber, $orderId);
			    if (!empty($transactionNumber)) {

			    	//Complete order when ticket is processed
			        $order->payment_complete($transactionNumber);

			        // ENABLE THIS WHEN STOCK NEEDS TO BE REDUCED
					//$order->reduce_order_stock();
					echo "<br>TransactionNumber: $transactionNumber";
					$note = "Order complete with transaction ID: $transactionNumber";
					$order->add_order_note( $note );
			    }
			} catch (Exception $e) {
			    // Error when processing the ticket
			    echo "<br>Error: - $e";
				$order->update_status('failed', sprintf(__('Error return code: %1$s'), $e));
				$note = "Order is failed, the error returned can be viewed below.";
				$order->add_order_note( $note );
			}

			// Regardless the process result we return an OK to Cyclos envrioment
			http_response_code(200);

			update_option('webhook_debug', $_GET);
			die();
		}

		//Function to be able to generate HTML for accessClient generator
		public function generate_button_html( $key, $data ) {
			$field    = $this->plugin_id . $this->id . '_' . $key;
			$defaults = array(
				'class'             => 'button-secondary',
				'css'               => '',
				'custom_attributes' => array(),
				'desc_tip'          => false,
				'description'       => '',
				'title'             => '',
			);

			$data = wp_parse_args( $data, $defaults );

			ob_start();
			?>
			<tr valign="top">
				<th scope="row" class="titledesc">
					<label for="<?php echo esc_attr( $field ); ?>"><?php echo wp_kses_post( $data['title'] ); ?></label>
					<?php echo $this->get_tooltip_html( $data ); ?>
				</th>
				<td class="forminp">
					<fieldset>
						<legend class="screen-reader-text"><span><?php echo wp_kses_post( $data['title'] ); ?></span></legend>
						<form method="get" name="accessClientForm" id="accessClientForm" action="">
							<input type="text" id="accessClientCode" name="accessClientCode" placeholder="AccessClient Activation Code">
							<button onclick="$("#accessClientForm").submit();" class="<?php echo esc_attr( $data['class'] ); ?>" type="submit" name="<?php echo esc_attr( $field ); ?>" id="<?php echo esc_attr( $field ); ?>" style="<?php echo esc_attr( $data['css'] ); ?>" <?php echo $this->get_custom_attribute_html( $data ); ?>>Get AccessClient</button>
							<p id="accessClientKey"class="description" style="color:red;"><?php echo $GLOBALS['GeneratedAccessClientToken']; ?></p>
							<p class="description"><u>Fill in your activation code and click "Get accessClient" to receive your own accessClient code </u><br> Activation code can be received by logging in at your Cyclos Payment Enviroment: <br> Persoonlijk > Webshop toegangscodes > Toevoegen > Enter a suitable name > Opslaan > Activatiecode > Bevestigen.</p>
							<?php echo $this->get_description_html( $data ); ?>
						</form>
					</fieldset>
				</td>
			</tr>
			<?php
			return ob_get_clean();
		}

		//Function for all the plugin options
 		public function init_form_fields(){
			$this->form_fields = array(
				'enabled' => array(
					'title'       => 'Enable/Disable',
					'label'       => 'Enable Cyclos Gateway',
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no'
				),
				'title' => array(
					'title'       => 'Title',
					'type'        => 'text',
					'description' => 'This controls the title which the user will see during checkout.',
					'default'     => 'United Economy',
					'desc_tip'    => true,
				),
				'description' => array(
					'title'       => 'Description',
					'type'        => 'textarea',
					'description' => 'This controls the description which the user will see during checkout.',
					'default'     => 'Betaal uw bestelling doormiddel via United Economy',
					'desc_tip'    => true,
				)
				,
				'accessclient_title' => array(
					'title'       => __( 'accessClient' ),
					'type'        => 'title',
					'description' => 'Define if the API accessClient is used or not.',
				),
				'accessclientmode' => array(
					'title'       => 'Enable / Disable accessClient mode',
					'type'        => 'checkbox',
					'description' => 'Tick the checkbox to enable accessClient mode, only accessClient API is available when enabled. username/password authentication will not be used when enabled. <br>accessClient mode is <b>recommended</b>',
				),
				'accessClientGenerate' => array(
					'type'        => 'button',
					'desc_tip'    => 'Root URL, API Username and API Password must be filled to get an accessClient code. If Testmode is active the Test API credentials will be used to generate the token. REMEMBER TO SAVE AFTER TOKEN HAS BEEN GENERATED!',
				),
				'testmode_title' => array(
					'title'       => __( 'Test mode'),
					'type'        => 'title',
					'description' => 'Enable or Disable test mode to test out the plugin. Enter the root URL, Test Username and Password before saving!',
				),
				'testmode' => array(
					'title'       => 'Enable/Disable test mode',
					'label'       => 'Test Mode',
					'type'        => 'checkbox',
					'description' => 'Place the payment gateway in test mode using test API keys.',
					'default'     => 'yes',
					'desc_tip'    => true,
				),
				'test_root_url' => array(
					'title'       => 'Test API Root URL',
					'type'        => 'text',
					'default'     => 'https://communities.cyclos.org/demo',
					'description' => 'Default Root URL is: "https://communities.cyclos.org/demo".',
					
				),
				'test_api_username' => array(
					'title'       => 'Test API Username',
					'type'        => 'text',
					'default'     => 'ticket',
					'description' => 'Default test username is: ticket.',
				),
				'test_api_password' => array(
					'title'       => 'Test API password',
					'type'        => 'password',
					'default'     => '1234',
					'description' => 'Default test password is: 1234',
				),
				'test_api_accessclient' => array(
					'title'       => 'Test API accessClient',
					'type'        => 'text',
				),
				'live_title' => array(
					'title'       => __( 'Live API'),
					'type'        => 'title',
					'description' => 'Enter the root URL, Test Username and Password of the live production API',
				),
				'root_url' => array(
					'title'       => 'Live API Root URL',
					'type'        => 'text',
				),
				'api_username' => array(
					'title'       => 'Live API Username',
					'type'        => 'text',
				),
				'api_password' => array(
					'title'       => 'Live API Password',
					'type'        => 'password',
				),
				'api_accessclient' => array(
					'title'       => 'Live API accessClient',
					'type'        => 'text',
				),
				'url_title' => array(
					'title'       => __( 'Return Pages'),
					'type'        => 'title',
					'description' => 'Define the full URLs of the return pages! If the fields are blank it will be the default Woocommerce return endpoints.<br><i>This option is not required and not recommended. Default Woocommerce endpoints can be found at:<br><a href="/wp-admin/admin.php?page=wc-settings&tab=advanced">Woocommerce settings > Advanced > Endpoints for checkout.</a></i>',
				),
				'succes_url' => array(
					'title'       => 'Succes URL',
					'type'        => 'text',
					'description' => 'Enter the full URL of the page where the customer needs to be returned to after a <b>succesfull</b> payment',
				),
				'cancel_url' => array(
					'title'       => 'Cancel URL',
					'type'        => 'text',
					'description' => 'Enter the full URL of the page where the customer needs to be returned to after a <b>cancelled</b> payment',
				)
			);
	 	}


		//Field Validation function during checkout


		//Admin options validation and processing.	

		public function process_admin_options(){
 			
			if ($_POST['woocommerce_cyclos_accessclientmode'] == true) {
				if ($_POST['woocommerce_cyclos_testmode'] == true) {
					if (empty($_POST['woocommerce_cyclos_test_api_accessclient'])){
						WC_Admin_Settings::add_error( 'Error: Test API accessClient not valid' );
						return false;
					} else {
						parent::process_admin_options();
						return true;
					}
				} else {
					if (empty($_POST['woocommerce_cyclos_api_accessclient'])){
						WC_Admin_Settings::add_error( 'Error: Live API accessClient not valid' );
						return false;
					} else {
						parent::process_admin_options();
						return true;
					}
				}
			} else {
				if ($_POST['woocommerce_cyclos_testmode'] == true) {
					if (empty($_POST['woocommerce_cyclos_test_api_username'])){
						WC_Admin_Settings::add_error( 'Error: Test API username not valid' );
						return false;
					} else if (empty($_POST['woocommerce_cyclos_test_api_password'])){
						WC_Admin_Settings::add_error( 'Error: Test API password not valid' );
						return false;
					} else {
						parent::process_admin_options();
						return true;
					}
				} else {
					if (empty($_POST['woocommerce_cyclos_api_username'])){
						WC_Admin_Settings::add_error( 'Error: Live API username not valid' );
						return false;
					} else if (empty($_POST['woocommerce_cyclos_api_password'])){
						WC_Admin_Settings::add_error( 'Error: Live API password not valid' );
						return false;
					} else {
						parent::process_admin_options();
						return true;
					}
				}
			}
		}

		//Checkout processing function
	 	public function process_payment( $order_id ) {
 			//Globalize Woocommerce
			global $woocommerce;

				//Get Shop Name for Order Description
				$shop_title = get_bloginfo('name');

				//Get Order ID
				$order = wc_get_order( $order_id );

				//Get Payment root for request
				$PAY_ROOT = "$this->root_url/pay";

				//Define order price amount
				$amount = $order->get_total();

				//Defining description with payment amount and shop name
				$description = "Betaling van $amount via de webshop: $shop_title";

				// A ticket could be created for a specific customer (in this example we do not use it, and in most cases this will not be used)
				$payer = null;

				// A ticket could have custom fields (not used)
				$customValues = new stdClass();

				// The default expiration is one day we changed to 1 hour
				$expiration = array(
				    'amount' => 1,
				    'field' => 'hours'
				);

				// Define WebshopRootURL
				$webshopRoot = curPageURL();

				//Define Payment Type
				$type = "handelsrekening.handels_transactie";

				//Ticket urls
				$successUrl = (empty($this->succes_url)) ? $order->get_checkout_order_received_url() : $this->succes_url;
				$successWebhookUrl = "$webshopRoot/wc-api/cyclos_payment_completed?orderId=$order_id?ticketNumber=$ticketNumber";
				$cancelUrl = (empty($this->cancel_url)) ? $order->get_cancel_order_url() : $this->cancel_url;

				//Specify test arguments for ticket creation without type argument
				$testargs = array(
					'amount' => $amount,
				    'description' => $description,
				    'payer' => $payer,
				    'successUrl' => $successUrl,
				    'successWebhook' => $successWebhookUrl,
				    'cancelUrl' => $cancelUrl,
				    'orderId' => $order_id,
				    'expiresAfter' => $expiration
				);
				//Specify live arguments for ticket creation without type argument
				$liveargs = array(
					'amount' => $amount,
				    'description' => $description,
				    'payer' => $payer,
				    'successUrl' => $successUrl,
				    'successWebhook' => $successWebhookUrl,
				    'cancelUrl' => $cancelUrl,
				    'orderId' => $order_id,
				    'type'=> $type,
				    'expiresAfter' => $expiration
				);

				//Specify Array for API interaction (Ticket request)
				$args = $this->testmode ? $testargs : $liveargs;


				//Definde ticket number
				$ticketNumber = create_ticket($args);
				$body = json_decode( 'body', true );
				
				//Check if create_ticket repsonse is error
				if (strpos($ticketNumber, 'Error') !== false) {
					//Add WC Notice with error message
					wc_add_notice($ticketNumber);
					return false;
				} else {
					//Return is succesfull, so redirection is taking place
					return array(
					'result' => 'success',
					'redirect' => $PAY_ROOT . '/' . $ticketNumber
					);
				}
		 
		} 

 	}
}



function process_ticket($ticketNumber, $orderId) {
    global $config;

    $request = curl_init("{$config['root']}/api/tickets/$ticketNumber/process?orderId=$orderId");
    
    curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($request, CURLOPT_POST, true);
    curl_setopt($request, CURLOPT_TIMEOUT, 30);

    add_headers($request);
    
    // Execute the request
    $response = curl_exec($request);
    check_response($request, $response);
    
    // Get the response
    $httpStatus = curl_getinfo($request, CURLINFO_HTTP_CODE);
    $result = json_decode($response);
    $errorMessage = null;
    
    switch ($httpStatus) {
        case 200:
            if ($result->actuallyProcessed) {
                $tx = $result->transaction;
                // Using the transaction number is preferred.
                // But in case it is disabled in Cyclos, return the internal identifier.
                return empty($tx->transactionNumber) ? $tx->id : $tx->transactionNumber; 
            }
            return NULL;
        case 401:
            $errorMessage = "Missing credentials";
            break;
        case 403:
            $errorMessage = "Permission denied";
            break;
        case 404:
            $errorMessage = "Ticket not found";
            break;
        case 422:
            $errorMessage = "Invalid ticket";
            break;
        case 500:
            // An error has occurred generating the payment
            if ($result->code == 'insufficientBalance') {
                $errorMessage = 'The customer does not have enough balance.';
            } else if ($result->code == 'destinationUpperLimitReached') {
                $errorMessage = 'You have reached the upper credit limit.';
            } else {
                // There are more error codes but for the sake of simplicity we only distinguish those two
                // Log a detailed error
                error_log("An unexpected error has occurred processing the ticket "
                . "(type = {$result-> exceptionType}, message = {$result->exceptionMessage})");
                $errorMessage = 'An unexpected error has occurred processing the ticket.';
            }
            break;
        default:
            $errorMessage = "An unknown error has occurred ($httpStatus)";
    }
    
    // There was an error
    throw new Exception($errorMessage);
}

/**
 * Adds required headers to send valid data and authenticate with Cyclos. 
 */
function add_headers($request) {
    global $config;
    $headers = array('Content-Type: application/json');
    if (empty($config['accessClient'])) {
        curl_setopt($request, CURLOPT_USERPWD, "{$config['user']}:{$config['password']}");
    } else {
        array_push($headers, "Access-Client-Token: {$config['accessClient']}");
    }
    curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
}

function check_response($request, $response) {
    if ($response === false) {
        $info = curl_getinfo($request);
        curl_close($request);
        echo 'Error performing request to Cyclos: ';
        echo "<pre>";
        print_r($info);
        echo "</pre>";
        die();
    }
}

function create_ticket($body){
    global $config;
    // Set CURL parameters required to create the ticket
    $request = curl_init("{$config['root']}/api/tickets");
    curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($request, CURLOPT_POST, true);
    curl_setopt($request, CURLOPT_POSTFIELDS, json_encode($body));
    curl_setopt($request, CURLOPT_TIMEOUT, 30);
    add_headers($request);
    
    // Execute the request
    $response = curl_exec($request);
    check_response($request, $response);
    
    // Get the response
    $http_status = curl_getinfo($request, CURLINFO_HTTP_CODE);
    if ($http_status === 201) {
        // The ticket was created.
        $ticket = json_decode($response);
        return $ticket->ticketNumber;
    } else {
        // There was an error
        $error = json_decode($response);
        return "Payment Method Error:<br> $http_status $error->code";
    }
}

function curPageURL() {
    $pageURL = 'http';
    if ($_SERVER["HTTPS"] == "on") {$pageURL .= "s";}
    $pageURL .= "://";
    if ($_SERVER["SERVER_PORT"] != "80") {
        $pageURL .= $_SERVER["SERVER_NAME"].":".$_SERVER["SERVER_PORT"].$_SERVER["REQUEST_URI"];
    } else {
        $pageURL .= $_SERVER["SERVER_NAME"].$_SERVER["REQUEST_URI"];
    }
    $path = pathinfo($pageURL);
    $webshopPath = $path['dirname'];
    return $webshopPath;
}

function get_accessclient($accesscode) {

	global $config;
    // Set CURL parameters required to create the accessclient token
    $request = curl_init("{$config['root']}/api/clients/activate?code=$accesscode");
    curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($request, CURLOPT_POST, true);
    curl_setopt($request, CURLOPT_POSTFIELDS, json_encode($body));
    curl_setopt($request, CURLOPT_TIMEOUT, 30);
    add_headers($request);
    
    // Execute the request
    $response = curl_exec($request);
    check_response($request, $response);
    
    // Get the response
    $http_status = curl_getinfo($request, CURLINFO_HTTP_CODE);
    if ($http_status === 200) {
        // The accessClient Token was created.
        $accessclient = json_decode($response);
        return $accessclient->token;
    } else {
        // There was an error
    	$error = json_decode($response);
        return "Error: $http_status, $error->entityType";
    }
}


?>